/**
 * 
 */
package x1.monitoring;

import java.io.File;
import java.lang.management.ClassLoadingMXBean;
import java.lang.management.CompilationMXBean;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.OperatingSystemMXBean;

import javax.swing.filechooser.FileSystemView;

/**
 * @author joe
 * 
 */
public class MonitoringServiceImpl implements MonitoringService {
  public MonitoringServiceImpl() {
    init();
  }

  private void init() {
    operatingSystemMXBean = ManagementFactory.getOperatingSystemMXBean();
    // ManagementFactory.getRuntimeMXBean();
    ManagementFactory.getThreadMXBean();
    classLoadingMXBean = ManagementFactory.getClassLoadingMXBean();
    compilationMXBean = ManagementFactory.getCompilationMXBean();
    // ManagementFactory.getGarbageCollectorMXBeans();
    memoryMXBean = ManagementFactory.getMemoryMXBean();
  }
  
  /*
   * (non-Javadoc)
   * @see x1.monitoring.MonitoringService#getSystemLoad()
   */
  @Override
  @SuppressWarnings("restriction")
  public SystemLoad getSystemLoad() {
    SystemLoad systemLoad = new SystemLoad();
    systemLoad.setSystemLoadAverage(operatingSystemMXBean.getSystemLoadAverage());
    systemLoad.setNumProcessors(operatingSystemMXBean.getAvailableProcessors());
    systemLoad.setName(operatingSystemMXBean.getName());
    systemLoad.setTimestamp(System.currentTimeMillis());
    systemLoad.setNanoTime(System.nanoTime());

    if (operatingSystemMXBean instanceof com.sun.management.OperatingSystemMXBean) {
      com.sun.management.OperatingSystemMXBean bean = (com.sun.management.OperatingSystemMXBean) operatingSystemMXBean;
      systemLoad.setProcessCpuTime(bean.getProcessCpuTime());
      if (lastSystemLoad != null) {
        int percent;
        if (systemLoad.getNanoTime() > lastSystemLoad.getNanoTime())
          percent = (int)(((systemLoad.getProcessCpuTime() - lastSystemLoad.getProcessCpuTime()) * 100L)
              / (systemLoad.getNanoTime() - lastSystemLoad.getNanoTime()));
        else
          percent = 0;
        systemLoad.setProcessCpuUsage(percent);
        /*
         * bean.getCommittedVirtualMemorySize();
         * bean.getFreePhysicalMemorySize(); bean.getFreeSwapSpaceSize();
         * bean.getProcessCpuTime(); bean.getTotalPhysicalMemorySize();
         * bean.getTotalSwapSpaceSize();
         */
      }
    }
    lastSystemLoad = systemLoad;
    return systemLoad;
  }

  /*
   * (non-Javadoc)
   * @see x1.monitoring.MonitoringService#getDiskUsage()
   */
  @Override
  public FilesystemUsage getDiskUsage() {
    File[] roots = FileSystemView.getFileSystemView().getRoots();
    FilesystemUsage fileSystemUsage = new FilesystemUsage();
    DiskUsage[] diskUsages = new DiskUsage[roots.length];
    int i = 0;
    for (File root : roots) {
      DiskUsage diskUsage = new DiskUsage();
      diskUsage.setName(root.getAbsolutePath().toString());
      diskUsage.setFreeSpace(root.getFreeSpace());
      diskUsage.setTotalSpace(root.getTotalSpace());
      diskUsage.setUsableSpace(root.getUsableSpace());     
      diskUsages[i++] = diskUsage;
    }
    fileSystemUsage.setDiskUsages(diskUsages);
    fileSystemUsage.setTimestamp(System.currentTimeMillis());
    return fileSystemUsage;
  }

  /*
   * (non-Javadoc)
   * @see x1.monitoring.MonitoringService#getClassUsage()
   */
  @Override
  public ClassUsage getClassUsage() {
    ClassUsage classUsage = new ClassUsage();
    classUsage.setLoadedClassCount(classLoadingMXBean.getLoadedClassCount());
    classUsage.setTotalLoadedClassCount(classLoadingMXBean.getTotalLoadedClassCount());
    classUsage.setUnloadedClassCount(classLoadingMXBean.getUnloadedClassCount());
    classUsage.setTotalCompilationTime(compilationMXBean.getTotalCompilationTime());
    classUsage.setName(compilationMXBean.getName());
    classUsage.setTimestamp(System.currentTimeMillis());
    return classUsage;
  }
  
  /*
   * (non-Javadoc)
   * @see x1.monitoring.MonitoringService#getMemoryUsage()
   */
  @Override
  public MemoryUsage getMemoryUsage() {
    MemoryUsage memoryUsage = new MemoryUsage();
    memoryUsage.setHeapMemoryUsageInit(memoryMXBean.getHeapMemoryUsage().getInit());
    memoryUsage.setHeapMemoryUsageCommitted(memoryMXBean.getHeapMemoryUsage().getCommitted());
    memoryUsage.setHeapMemoryUsageUsed(memoryMXBean.getHeapMemoryUsage().getUsed());
    memoryUsage.setHeapMemoryUsageMax(memoryMXBean.getHeapMemoryUsage().getMax());
    memoryUsage.setNonHeapMemoryUsageInit(memoryMXBean.getNonHeapMemoryUsage().getInit());
    memoryUsage.setNonHeapMemoryUsageCommitted(memoryMXBean.getNonHeapMemoryUsage().getCommitted());
    memoryUsage.setNonHeapMemoryUsageUsed(memoryMXBean.getNonHeapMemoryUsage().getUsed());
    memoryUsage.setNonHeapMemoryUsageMax(memoryMXBean.getNonHeapMemoryUsage().getMax());
    memoryUsage.setObjectPendingFinalizationCount(memoryMXBean.getObjectPendingFinalizationCount());
    memoryUsage.setTimestamp(System.currentTimeMillis());
    return memoryUsage;
  }
  
  private SystemLoad lastSystemLoad;
  private OperatingSystemMXBean operatingSystemMXBean;
  private ClassLoadingMXBean classLoadingMXBean;
  private CompilationMXBean compilationMXBean;
  private MemoryMXBean memoryMXBean;
}
