/**
 * 
 */
package x1.monitoring;

import java.io.StringWriter;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.jackson.map.ObjectMapper;
import org.eclipse.jetty.websocket.WebSocket;

/**
 * @author joe
 * 
 */
public class MonitoringSocket implements WebSocket {
  private static final Log log = LogFactory.getLog(MonitoringSocket.class);
  private MonitoringService service;
  private Outbound outbound;
  private Timer timer;
  private ObjectMapper mapper;

  public MonitoringSocket(MonitoringService service) {
    this.service = service;
    this.mapper = new ObjectMapper();
  }

  /**
   * @return the service
   */
  public MonitoringService getService() {
    return service;
  }

  /**
   * @param service
   *          the service to set
   */
  public void setService(MonitoringService service) {
    this.service = service;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.jetty.websocket.WebSocket#onConnect(org.eclipse.jetty.websocket
   * .WebSocket.Outbound)
   */
  @Override
  public void onConnect(Outbound outbound) {
    this.outbound = outbound;
    timer = new Timer();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.jetty.websocket.WebSocket#onDisconnect()
   */
  @Override
  public void onDisconnect() {
    timer.cancel();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.jetty.websocket.WebSocket#onMessage(byte,
   * java.lang.String)
   */
  public void onMessage(final byte frame, String data) {
    abstract class MonitoringTimerTask extends TimerTask {
      @Override      
      public void run() {
        try {
          MonitoringValue value = getValue();
          outbound.sendMessage(frame, stringify(value));
        } catch (Exception e) {
          log.error(null, e);
        }
      }
      
      protected abstract MonitoringValue getValue();
    }
    
    log.info("Incoming: " + data);
    if (data.indexOf("disconnect") >= 0) {
      outbound.disconnect();
    } else if (data.indexOf("connect") >= 0) {
      timer.schedule(new MonitoringTimerTask() {
        @Override
        public MonitoringValue getValue() {
          return service.getSystemLoad();
        }
      }, new Date(), 2000);
      timer.schedule(new MonitoringTimerTask() {
        @Override
        public MonitoringValue getValue() {
          return service.getClassUsage();
        }
      }, new Date(), 5000);
      timer.schedule(new MonitoringTimerTask() {
        @Override
        public MonitoringValue getValue() {
          return service.getDiskUsage();
        }
      }, new Date(), 10000);
      timer.schedule(new MonitoringTimerTask() {
        @Override
        public MonitoringValue getValue() {
          return service.getMemoryUsage();
        }
      }, new Date(), 3000);
    }
  }

  private String stringify(Object value) throws Exception {
    StringWriter w = new StringWriter();
    mapper.writeValue(w, value);
    return w.toString();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.jetty.websocket.WebSocket#onMessage(byte, byte[], int,
   * int)
   */
  @Override
  public void onMessage(byte arg0, byte[] arg1, int arg2, int arg3) {
    // TODO Auto-generated method stub
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.jetty.websocket.WebSocket#onFragment(boolean, byte,
   * byte[], int, int)
   */
  @Override
  public void onFragment(boolean arg0, byte arg1, byte[] arg2, int arg3, int arg4) {
    // TODO Auto-generated method stub
  }
}
