var data = {
		'os': { color: '#FF1C0A', min: 0, max: 100, values: []},
		'class': { color: '#FFFD0A', min: 0, max: 1000, values: []},
		'memory': { color: '#00A308', min: 0, max: 128 * 1024 * 1024, values: []},
		'disk': { color: '#0008DB', min: 0, max: 100 * 1024 * 1024 * 1024, values: []}
};
var width = 150;
var height = 150;
var minutes = 5;

function draw(id) {
	var canvas = document.getElementById('graph-' + id);
	if (canvas.getContext) {
		var ctx = canvas.getContext('2d');
		ctx.clearRect(0,0,width,height);
		drawGrid(id, ctx);
		drawData(id, ctx);
	}
}

function pushData(id, timestamp, value) {
	var d = data[id];
	if (!d) {
		data[id] = d;
	}
	if (value > d.max) {
		d.max = value * 2;
	}
	var minTime = timestamp - minutes * 60 * 1000;
	var skipIndex = -1;
	for (var i=0; i<d.values.length; i++) {
		if (d.values[i].timestamp < minTime) {
			skipIndex = i;
		}
	}
	if (skipIndex >= 0) {
		var newValues = [];
		for (i=skipIndex + 1; i<d.values.length; i++) {
			newValues[i - skipIndex - 1] = d.values[i];
		}
		d.values = newValues;
	}
	d.values[d.values.length] = {'timestamp': timestamp, 'value': value};
}

function drawGrid(id, ctx) {
	var stepX = width / minutes;
	
	ctx.fillStyle = 'rgb(0,0,0)';
	ctx.fillText(id, 2, 10);
	ctx.strokeStyle = 'rgb(16,16,16)';
	ctx.lineWidth = 1;
	ctx.beginPath();
	for ( var x = 0; x <= width; x += stepX) {
		ctx.moveTo(x, height);
		ctx.lineTo(x, height - 5);
	}
	ctx.stroke();
	var stepY = 20;
	ctx.strokeStyle = 'rgb(192,192,192)';
	ctx.lineWidth = 1;
	ctx.beginPath();
	for ( var y = stepY; y < height; y += stepY) {
		ctx.moveTo(0, height - y);
		ctx.lineTo(width, height - y);
	}
	ctx.stroke();
}

function drawData(id, ctx) {
	var d = data[id];
	if (d.values.length==0)
		return;
	var stepX = width / minutes / 60 / 1000;
	var stepY = height / (d.max - d.min);
	ctx.strokeStyle = d.color;
	ctx.lineWidth = 2;
	ctx.beginPath();
	var minTime = d.values[0].timestamp;
	for (var i=0; i<d.values.length; i++) {
		var x = (d.values[i].timestamp - minTime) * stepX;
		var y = (d.values[i].value) * stepY;
		ctx.lineTo(x, height - y);		
	}
	ctx.stroke();
}